@@ -16,11 +16,14 @@
  * Licensed under GPLv2 or later, see file LICENSE in this tarball for details.
  */
 
-/* These defines are very important for BusyBox.  Without these,
- * huge chunks of ram are pre-allocated making the BusyBox bss
- * size Freaking Huge(tm), which is a bad thing.*/
+/* TODO: full support for -v for DESKTOP
+/usr/bin/gzip -v a bogus aa
+a:       85.1% -- replaced with a.gz
+gzip: bogus: No such file or directory
+aa:      85.1% -- replaced with aa.gz
+*/
+
 #define SMALL_MEM
-#define DYN_ALLOC
 
 #include <stdlib.h>
 #include <stdio.h>
@@ -87,17 +90,11 @@ typedef unsigned long ulg;
 #  endif
 #endif
 
-#ifdef DYN_ALLOC
 #  define DECLARE(type, array, size)  static type * array
 #  define ALLOC(type, array, size) { \
-      array = (type*)xcalloc((size_t)(((size)+1L)/2), 2*sizeof(type)); \
+      array = (type*)xzalloc((size_t)(((size)+1L)/2) * 2*sizeof(type)); \
    }
 #  define FREE(array) {free(array), array=NULL;}
-#else
-#  define DECLARE(type, array, size)  static type array[size]
-#  define ALLOC(type, array, size)
-#  define FREE(array)
-#endif
 
 #define tab_suffix window
 #define tab_prefix prev	/* hash link (see deflate.c) */
@@ -133,8 +130,8 @@ typedef int file_t;		/* Do not use stdio */
 #define ASCII   1
 
 #ifndef WSIZE
-#  define WSIZE 0x8000	/* window size--must be a power of two, and */
-#endif							/*  at least 32K for zip's deflate method */
+#  define WSIZE 0x8000  /* window size--must be a power of two, and */
+#endif                  /*  at least 32K for zip's deflate method */
 
 #define MIN_MATCH  3
 #define MAX_MATCH  258
@@ -154,15 +151,6 @@ typedef int file_t;		/* Do not use stdio */
 #define put_byte(c) {outbuf[outcnt++]=(uch)(c); if (outcnt==OUTBUFSIZ)\
    flush_outbuf();}
 
-
-/* Output a 32 bit value to the bit stream, lsb first */
-#if 0
-#define put_long(n) { \
-    put_short((n) & 0xffff); \
-    put_short(((ulg)(n)) >> 16); \
-}
-#endif
-
 #define seekable()    0	/* force sequential output */
 #define translate_eol 0	/* no option -a yet */
 
@@ -282,17 +270,17 @@ DECLARE(ush, tab_prefix, 1L << BITS);
 static int foreground;	/* set if program run in foreground */
 static int method = DEFLATED;	/* compression method */
 static int exit_code = OK;	/* program exit code */
-static int part_nb;		/* number of parts in .gz file */
 static long time_stamp;	/* original time stamp (modification time) */
-static long ifile_size;	/* input file size, -1 for devices (debug only) */
 static char z_suffix[MAX_SUFFIX + 1];	/* default suffix (can be set with --suffix) */
-static int z_len;		/* strlen(z_suffix) */
 
 static int ifd;			/* input file descriptor */
 static int ofd;			/* output file descriptor */
+#ifdef DEBUG
 static unsigned insize;	/* valid bytes in inbuf */
+#endif
 static unsigned outcnt;	/* bytes in output buffer */
 
+static uint32_t *crc_32_tab;
 
 /* Output a 16 bit value, lsb first */
 static void put_short(ush w)
@@ -320,7 +308,9 @@ static void abort_gzip(int ATTRIBUTE_UNUSED ignored)
 static void clear_bufs(void)
 {
 	outcnt = 0;
+#ifdef DEBUG
 	insize = 0;
+#endif
 	bytes_in = 0L;
 }
 
@@ -333,7 +323,7 @@ static void write_buf(int fd, void *buf, unsigned cnt)
 	unsigned n;
 
 	while ((n = write(fd, buf, cnt)) != cnt) {
-		if (n == (unsigned) (-1)) bb_error_msg_and_die("can't write");
+		if (n == (unsigned) (-1)) bb_error_msg_and_die(bb_msg_write_error);
 		cnt -= n;
 		buf = (void *) ((char *) buf + n);
 	}
@@ -344,32 +334,13 @@ static void write_buf(int fd, void *buf, unsigned cnt)
  * pointer, then initialize the crc shift register contents instead.
  * Return the current crc in either case.
  */
-static ulg updcrc(uch * s, unsigned n)
+static uint32_t updcrc(uch * s, unsigned n)
 {
-	static ulg crc = (ulg) 0xffffffffL;	/* shift register contents */
-	register ulg c;		/* temporary variable */
-	static unsigned long crc_32_tab[256];
-
-	if (crc_32_tab[1] == 0x00000000L) {
-		unsigned long csr;	/* crc shift register */
-		const unsigned long e = 0xedb88320L;	/* polynomial exclusive-or pattern */
-		int i;			/* counter for all possible eight bit values */
-		int k;			/* byte being shifted into crc apparatus */
-
-		/* Compute table of CRC's. */
-		for (i = 1; i < 256; i++) {
-			csr = i;
-			/* The idea to initialize the register with the byte instead of
-			   * zero was stolen from Haruhiko Okumura's ar002
-			 */
-			for (k = 8; k; k--)
-				csr = csr & 1 ? (csr >> 1) ^ e : csr >> 1;
-			crc_32_tab[i] = csr;
-		}
-	}
+	static uint32_t crc = ~0;	/* shift register contents */
+	uint32_t c;		/* temporary variable */
 
 	if (s == NULL) {
-		c = 0xffffffffL;
+		c = ~0;
 	} else {
 		c = crc;
 		if (n)
@@ -378,7 +349,7 @@ static ulg updcrc(uch * s, unsigned n)
 			} while (--n);
 	}
 	crc = c;
-	return c ^ 0xffffffffL;	/* (instead of ~c for 64-bit machines) */
+	return ~c;
 }
 
 /* bits.c -- output variable-length bit strings
@@ -512,7 +483,7 @@ static void send_bits(int value, int length)
  */
 static unsigned bi_reverse(unsigned code, int len)
 {
-	register unsigned res = 0;
+	unsigned res = 0;
 
 	do {
 		res |= code & 1;
@@ -810,7 +781,7 @@ static void check_match(IPos start, IPos match, int length);
  */
 static void lm_init(ush * flags)
 {
-	register unsigned j;
+	unsigned j;
 
 	/* Initialize the hash table. */
 	memset(head, 0, HASH_SIZE * sizeof(*head));
@@ -860,9 +831,9 @@ static void lm_init(ush * flags)
 static int longest_match(IPos cur_match)
 {
 	unsigned chain_length = max_chain_length;	/* max hash chain length */
-	register uch *scan = window + strstart;	/* current string */
-	register uch *match;	/* matched string */
-	register int len;	/* length of current match */
+	uch *scan = window + strstart;	/* current string */
+	uch *match;	/* matched string */
+	int len;	/* length of current match */
 	int best_len = prev_length;	/* best match length so far */
 	IPos limit =
 		strstart > (IPos) MAX_DIST ? strstart - (IPos) MAX_DIST : NIL;
@@ -876,9 +847,9 @@ static int longest_match(IPos cur_match)
 #if HASH_BITS < 8 || MAX_MATCH != 258
 #  error Code too clever
 #endif
-	register uch *strend = window + strstart + MAX_MATCH;
-	register uch scan_end1 = scan[best_len - 1];
-	register uch scan_end = scan[best_len];
+	uch *strend = window + strstart + MAX_MATCH;
+	uch scan_end1 = scan[best_len - 1];
+	uch scan_end = scan[best_len];
 
 	/* Do not waste too much time if we already have a good match: */
 	if (prev_length >= good_match) {
@@ -965,7 +936,7 @@ static void check_match(IPos start, IPos match, int length)
  */
 static void fill_window(void)
 {
-	register unsigned n, m;
+	unsigned n, m;
 	unsigned more =
 		(unsigned) (window_size - (ulg) lookahead - (ulg) strstart);
 	/* Amount of free space at the end of the window. */
@@ -1033,7 +1004,7 @@ static ulg deflate(void)
 	IPos prev_match;	/* previous match */
 	int flush;			/* set if current block must be flushed */
 	int match_available = 0;	/* set if previous match exists */
-	register unsigned match_length = MIN_MATCH - 1;	/* length of best match */
+	unsigned match_length = MIN_MATCH - 1;	/* length of best match */
 
 	/* Process the input block. */
 	while (lookahead != 0) {
@@ -1158,44 +1129,37 @@ typedef struct dirent dir_type;
 /* ======================================================================== */
 int gzip_main(int argc, char **argv)
 {
+	enum {
+		OPT_tostdout = 0x1,
+		OPT_force = 0x2,
+	};
+
+	unsigned opt;
 	int result;
 	int inFileNum;
 	int outFileNum;
 	struct stat statBuf;
 	char *delFileName;
-	int tostdout = 0;
-	int force = 0;
-	int opt;
 
-	while ((opt = getopt(argc, argv, "cf123456789dq")) != -1) {
-		switch (opt) {
-		case 'c':
-			tostdout = 1;
-			break;
-		case 'f':
-			force = 1;
-			break;
-			/* Ignore 1-9 (compression level) options */
-		case '1':
-		case '2':
-		case '3':
-		case '4':
-		case '5':
-		case '6':
-		case '7':
-		case '8':
-		case '9':
-			break;
-		case 'q':
-			break;
-#ifdef CONFIG_GUNZIP
-		case 'd':
-			optind = 1;
-			return gunzip_main(argc, argv);
-#endif
-		default:
-			bb_show_usage();
-		}
+	opt = getopt32(argc, argv, "cf123456789qv" USE_GUNZIP("d"));
+	//if (opt & 0x1) // -c
+	//if (opt & 0x2) // -f
+	/* Ignore 1-9 (compression level) options */
+	//if (opt & 0x4) // -1
+	//if (opt & 0x8) // -2
+	//if (opt & 0x10) // -3
+	//if (opt & 0x20) // -4
+	//if (opt & 0x40) // -5
+	//if (opt & 0x80) // -6
+	//if (opt & 0x100) // -7
+	//if (opt & 0x200) // -8
+	//if (opt & 0x400) // -9
+	//if (opt & 0x800) // -q
+	//if (opt & 0x1000) // -v
+	if (ENABLE_GUNZIP && (opt & 0x2000)) { // -d
+		/* FIXME: getopt32 should not depend on optind */
+		optind = 1;
+		return gunzip_main(argc, argv);
 	}
 
 	foreground = signal(SIGINT, SIG_IGN) != SIG_IGN;
@@ -1214,7 +1178,6 @@ int gzip_main(int argc, char **argv)
 #endif
 
 	strncpy(z_suffix, Z_SUFFIX, sizeof(z_suffix) - 1);
-	z_len = strlen(z_suffix);
 
 	/* Allocate all global buffers (for DYN_ALLOC option) */
 	ALLOC(uch, inbuf, INBUFSIZ + INBUF_EXTRA);
@@ -1223,12 +1186,13 @@ int gzip_main(int argc, char **argv)
 	ALLOC(uch, window, 2L * WSIZE);
 	ALLOC(ush, tab_prefix, 1L << BITS);
 
+	/* Initialise the CRC32 table */
+	crc_32_tab = crc32_filltable(0);
+
 	clear_bufs();
-	part_nb = 0;
 
 	if (optind == argc) {
 		time_stamp = 0;
-		ifile_size = -1L;
 		zip(STDIN_FILENO, STDOUT_FILENO);
 	} else {
 		int i;
@@ -1239,20 +1203,16 @@ int gzip_main(int argc, char **argv)
 			clear_bufs();
 			if (strcmp(argv[i], "-") == 0) {
 				time_stamp = 0;
-				ifile_size = -1L;
 				inFileNum = STDIN_FILENO;
 				outFileNum = STDOUT_FILENO;
 			} else {
-				inFileNum = open(argv[i], O_RDONLY);
-				if (inFileNum < 0 || fstat(inFileNum, &statBuf) < 0)
+				inFileNum = xopen(argv[i], O_RDONLY);
+				if (fstat(inFileNum, &statBuf) < 0)
 					bb_perror_msg_and_die("%s", argv[i]);
 				time_stamp = statBuf.st_ctime;
-				ifile_size = statBuf.st_size;
 
-				if (!tostdout) {
-					path = xmalloc(strlen(argv[i]) + 4);
-					strcpy(path, argv[i]);
-					strcat(path, ".gz");
+				if (!(opt & OPT_tostdout)) {
+					path = xasprintf("%s.gz", argv[i]);
 
 					/* Open output file */
 #if (__GLIBC__ >= 2) && (__GLIBC_MINOR__ >= 1) && defined O_NOFOLLOW
@@ -1273,7 +1233,7 @@ int gzip_main(int argc, char **argv)
 					outFileNum = STDOUT_FILENO;
 			}
 
-			if (path == NULL && isatty(outFileNum) && force == 0) {
+			if (path == NULL && isatty(outFileNum) && !(opt & OPT_force)) {
 				bb_error_msg
 					("compressed data not written to a terminal. Use -f to force compression.");
 				free(path);
@@ -1300,7 +1260,7 @@ int gzip_main(int argc, char **argv)
 		}
 	}
 
-	return (exit_code);
+	return exit_code;
 }
 
 /* trees.c -- output deflated data using Huffman coding
@@ -2416,7 +2376,7 @@ static void set_file_type(void)
  */
 
 
-static ulg crc;			/* crc on uncompressed file data */
+static uint32_t crc;			/* crc on uncompressed file data */
 static long header_bytes;	/* number of bytes in gzip header */
 
 static void put_long(ulg n)
@@ -2447,7 +2407,6 @@ static int zip(int in, int out)
 
 	/* Write the header to the gzip file. See algorithm.doc for the format */
 
-
 	method = DEFLATED;
 	put_header_byte(GZIP_MAGIC[0]);	/* magic header */
 	put_header_byte(GZIP_MAGIC[1]);
